'---------------------------------------------------------------------------------
' Golfed Rogue
' https://codegolf.stackexchange.com/questions/7074/build-a-simple-roguelike
' also https://github.com/nrkn/SimpleRLGolf
' from a rec.games.roguelike.development post
' DLosc QBASIC solution posted on StackExchange on 21APL2015
' licenced under https://creativecommons.org/licenses/by-sa/3.0/
' modified and commented for CMM2 by Womble 26OCT2020
' v1.1 Tested under Firmware v5.05.06B15
'---------------------------------------------------------------------------------

'Set array option base to index from 0
OPTION BASE 0

MODE 2                             '640x400 resolution, this plays nice with gLOCATE

'clear the screen to avoid unwanted scrolling from the CMM2 user interace
CLS

DIM w(12,10)                       'Binary Wall Map Array, initially everything is 0 a wall

'Print the Map
COLOUR RGB(BROWN)
FOR r=1 TO 12                      'map row r
   READ d                          'Read map row data
   FOR c=1 TO 10                   'map column c  
      w(r,c) = (d*2 AND 2^c) > 0   'Decode into binary array 1 is a space, 0 is a wall
      PRINT CHR$(35-3*w(r,c));     'Print map with char35 # for walls and char32 for spaces
   NEXT c
   'PRINT d;                       'Debug dislay of decimal Map Data for that row
   PRINT                           'This prints a carriage return
NEXT r

'Print the user instructions
COLOUR RGB(WHITE)
gLOCATE 18,1 : PRINT "Arrow Keys to Move, TAB or ESC to exit"

i=2                                'Start row for @ character
j=2                                'Start column for @ character

MainLoop: 
DO
   gLOCATE i,j
   COLOUR RGB(RED) : PRINT "@"     'Draw the @ character
   a$=""                 'Clear keyboard input string
   do while a$ = ""
      a$ = INKEY$        'Wait for keyboard input
   loop

   k=ASC(RIGHT$(a$,1))   'Keypress up 128, down 239, left 130, right 131 or TAB 9 ESC 27


   x=i+(k=129)-(k=128)   'Where are we trying to move to?
   y=j+(k=131)-(k=130)   'current location +1 +0 depending on k=left? + k=right? truth test

   COLOUR RGB(WHITE)
   gLOCATE 18,1 : PRINT "Arrow Keys to Move, TAB or ESC to exit"
   gLOCATE 20,1 : PRINT "                                         "       'Clear Debug Info
   gLOCATE 20,1 : PRINT "a$="a$, "keycode="k, "dest="x" x"y, "map="w(x,y) 'Print Debug Info

   gLOCATE i,j           'locate cursor to old @ location
   PRINT " "             'erase the old @ character

   IF w(x,y) THEN        'if new location is 1 a space ie. TRUE
      i=x                'update the location for the @ character 
      j=y                'otherwise location remains the same
   END IF

LOOP UNTIL k=9 OR k=27   'TAB or ESC key was pressed so exit


END                      'Exit the program gracefully


'---------------------------------------------------------------------------------

'Map Data (a binary string per row, 1 is space, 0 is wall)
DATA 48,438,390,510,252,765,765,252,510,390,438,48
'
'Map
'
'    ####  ####  =   48
'    #  #  #  #  =  438
'    #  ####  #  =  390
'    #        #  =  510
'    ##      ##  =  252
'     #      #   =  765
'     #      #   =  765
'    ##      ##  =  252
'    #        #  =  510
'    #  ####  #  =  390
'    #  #  #  #  =  438
'    ####  ####  =   48
'
'
'---------------------------------------------------------------------------------

SUB gLOCATE row%, col%
  LOCAL x% = col% -1                                           'Columns start at 1 
  LOCAL y% = row% -1                                           'Rows start at 1
  IF x% < 0 OR x% > ( MM.HRES \ MM.INFO(FONTWIDTH) ) THEN      'max character in row
    x% = MM.HRES \ MM.INFO(FONTWIDTH)                          'set to max characters
  ENDIF
  IF y% < 0 OR y% > ( MM.VRES \ MM.INFO(FONTHEIGHT) ) THEN     'max rows onscreen
    y% = MM.VRES \ MM.INFO(FONTHEIGHT)                         'set to max rows
  ENDIF
  PRINT @((MM.INFO(FONTWIDTH)*x%),(MM.INFO(FONTHEIGHT)*y%))""; 'position cursor
  'where x is the character column, and y is the character row
  'https://www.thebackshed.com/forum/ViewTopic.phpPRINT TID=12778&PID=155313#155313#155313
END SUB

'---------------------------------------------------------------------------------




                                                                                                      